#define AI_SCRIPT
#define AI_PLATOON_EC

platoon "AIPlatoon"
{

////    Declarations    ////

state Initialize;
state Nothing;
state StartMoving;
state Moving;
state StartWaitForEndAttacking;
state Attacking;

int m_nMoveMode;
int m_nMoveToGx;
int m_nMoveToGy;
int m_nMoveToLz;
int m_bDefendUnitsInPlatoon;

consts {
    eMoving    = 0x0001;
    eAttacking = 0x0002;
}
    
#include "..\Common.ech"
#include "..\Trace.ech"

////    Functions    ////

function void PlatoonAttack(unit uTarget)
{
    int nIndex, nCnt;
    punit uMember;

    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        uMember = GetUnit(nIndex);
        uMember.CommandAttack(uTarget);
    }
}//|

function void PlatoonStop()
{
    int nIndex, nCnt;
    punit uMember;

    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        uMember = GetUnit(nIndex);
        uMember.CommandStop();
    }
}//|

function void PlatoonMove(int nGx, int nGy, int nLz)
{
    m_nMoveToGx = nGx;
    m_nMoveToGy = nGy;
    m_nMoveToLz = nLz;
    CommandMovePlatoon(nGx, nGy, nLz, m_nMoveMode, 0);
}//|

function void PlatoonMove(int nGx, int nGy, int nLz, int nUnitsMask)
{
    m_nMoveToGx = nGx;
    m_nMoveToGy = nGy;
    m_nMoveToLz = nLz;
    CommandMovePlatoon(nGx, nGy, nLz, m_nMoveMode, nUnitsMask);
}//|

function void PlatoonMoveToFormation()
{
    punit uLeader;

    if (GetUnitsCount() > 0)
    {
        uLeader = GetUnit(0);
        ASSERT(uLeader != null);
        m_nMoveToGx = uLeader.GetLocationX();
        m_nMoveToGy = uLeader.GetLocationY();
        m_nMoveToLz = uLeader.GetLocationZ();
        CommandMovePlatoon(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz, movePlatoonFormation, 0);
    }
}//|

function void PlatoonSetMovementMode(int nMode)
{
    int nIndex, nCnt;
    punit uMember;

    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        uMember = GetUnit(nIndex);
        uMember.CommandSetMovementMode(nMode);
    }
}//|

function void SetFlagForAll(int nFlag)
{
    int nIndex, nCnt;

    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        SetUnitData(nIndex, GetUnitData(nIndex) | nFlag);
    }
}//|

function void SetAllMoveFlag()
{
    SetFlagForAll(eMoving);
}//|

function void SetAllAttackFlag()
{
    SetFlagForAll(eAttacking);
}//|

function int IsExecutingMovingCommand()
{
    int nIndex, nCnt;
    int bIsAnyMoving;
    int nData;
    punit uMember;

    bIsAnyMoving = false;
    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        nData = GetUnitData(nIndex);
        if (nData & eMoving)
        {
            uMember = GetUnit(nIndex);
            if (uMember.IsMoving())
            {
                bIsAnyMoving = true;
            }
            else
            {
                SetUnitData(nIndex, nData & ~eMoving);
            }
        }
    }
    return bIsAnyMoving;
}//|

function int IsExecutingAttackCommand()
{
    int nIndex, nCnt;
    int bHaveAnyTarget;
    int nData;
    punit uMember;

    bHaveAnyTarget = false;
    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        nData = GetUnitData(nIndex);
        if (nData & eAttacking)
        {
            uMember = GetUnit(nIndex);
            if (uMember.SetEventOnGetTarget() != null)
            {
                bHaveAnyTarget = true;
            }
            else
            {
                SetUnitData(nIndex, nData & ~eAttacking);
            }
        }
    }
    return bHaveAnyTarget;
}//|

//wywolywane po dodaniu nowego unita (z produkcji)
function void OnAddNewUnitToPlatoon(punit uUnit)
{
    punit uLeader;
    int nMoveToGx, nMoveToGy, nMoveToLz;
    int nMinCnt;

    ASSERT(state != Initialize);
    if ((state == StartMoving) || (state == Moving))
    {
        uUnit.CommandMove(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SetUnitData(uUnit, eMoving);
    }
    else if ((state == StartWaitForEndAttacking) || (state == Attacking) || (state == Nothing))
    {
        if (state == Nothing)
        {
            nMinCnt = 0;
        }
        else
        {
            //attacking
            nMinCnt = 4;
        }
        if (GetUnitsCount() > nMinCnt)
        {
            uLeader = GetUnit(0);
            if (uLeader.IsMoving())
            {
                nMoveToGx = uLeader.GetMoveTargetX();
                nMoveToGy = uLeader.GetMoveTargetY();
                nMoveToLz = uLeader.GetMoveTargetZ();
                uUnit.CommandMove(nMoveToGx, nMoveToGy, nMoveToLz);
            }
            else
            {
                nMoveToGx = uLeader.GetLocationX();
                nMoveToGy = uLeader.GetLocationY();
                nMoveToLz = uLeader.GetLocationZ();
            }
            if ((uUnit.DistanceTo(nMoveToGx, nMoveToGy) > 5) || (uUnit.GetLocationZ() != nMoveToLz))
            {
                uUnit.CommandMove(nMoveToGx, nMoveToGy, nMoveToLz);
            }
        }
    }
}//|

//funkcja ktora sprawdza czy jakis unit jest atakowany i jesli tak i jest unit w plutonie ktory
//nic nie robi to wysyla go aby bronil tamtego unita (wysyla go w jego kierunku)
//funkcja wprowadzona po to aby nie bylo tak, ze bija jakiegos naszego unita a inne stoja od niego za daleko
//i nic nie widza wiec stoja bezczynnie
function void DefendUnitsInPlatoon()
{
    punit arrAttackedUnits[];
    punit arrFreeUnits[];
    punit uUnit;
    unit uAttacker;
    int nIndex, nIndex2, nCnt, nCnt2, nStep, nSteps;
    int nPosGx, nPosGy, nPosLz;
    int nFreePerAttacked, nExtraFree;

    arrAttackedUnits.Create(0);
    arrFreeUnits.Create(0);
    nCnt = GetUnitsCount();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        uUnit = GetUnit(nIndex);
        uAttacker = uUnit.GetLatestAttacker();
        if (uAttacker != null)
        {
            arrAttackedUnits.Add(uUnit);
        }
        else
        {
            if (!uUnit.IsExecutingAnyCommand() && (uUnit.SetEventOnGetTarget() == null))
            {
                arrFreeUnits.Add(uUnit);
            }
        }
    }
    if ((arrAttackedUnits.GetSize() == 0) || (arrFreeUnits.GetSize() == 0))
    {
        return;
    }
    //do kolejnych unitow z arrAttackedUnits wysylamy unity z arrFreeUnits
    //nie sprawdzamy odleglosci (wysylania do bedacych najblizej) bo zajelo by to za duzo czasu
    nFreePerAttacked = arrFreeUnits.GetSize()/arrAttackedUnits.GetSize();
    if (nFreePerAttacked == 0)
    {
        nFreePerAttacked = 1;
    }
    nExtraFree = arrFreeUnits.GetSize()%arrAttackedUnits.GetSize();
    nCnt = arrAttackedUnits.GetSize();
    nIndex2 = 0;
    nCnt2 = arrFreeUnits.GetSize();
    for (nIndex = 0; nIndex < nCnt; ++nIndex)
    {
        uUnit = arrAttackedUnits[nIndex];
        nPosGx = uUnit.GetLocationX();
        nPosGy = uUnit.GetLocationY();
        nPosLz = uUnit.GetLocationZ();
        if (nIndex < nExtraFree)
        {
            nSteps = nFreePerAttacked + 1;
        }
        else
        {
            nSteps = nFreePerAttacked;
        }
        for (nStep = 0; (nIndex2 < nCnt2) && (nStep < nSteps); ++nStep)
        {
            uUnit = arrFreeUnits[nIndex2];
            uUnit.CommandMove(nPosGx, nPosGy, nPosLz);
            ++nIndex2;
        }
    }
}//|

////    States    ////

state Initialize
{
    return Nothing;
}//|

state Nothing
{
    if (m_bDefendUnitsInPlatoon)
    {
        DefendUnitsInPlatoon();
    }
    return Nothing;
}//|

state StartMoving
{
    return Moving;
}//|

state Moving
{
    if (m_bDefendUnitsInPlatoon)
    {
        DefendUnitsInPlatoon();
    }
    if (IsExecutingMovingCommand())
    {
        return Moving;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

state StartWaitForEndAttacking
{
    if (m_bDefendUnitsInPlatoon)
    {
        DefendUnitsInPlatoon();
    }
    return Attacking, 100;
}//|

state Attacking
{
    if (m_bDefendUnitsInPlatoon)
    {
        DefendUnitsInPlatoon();
    }
    if (IsExecutingAttackCommand())
    {
        return Attacking;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

////    Commands    ////

command Initialize()
{
    m_nMoveMode = moveSquareFormation;
    m_bDefendUnitsInPlatoon = false;
#ifdef USE_TRACE
    traceMode = 1;
#endif USE_TRACE
    return true;
}//|

command Uninitialize()
{
    //wykasowac referencje
    return true;
}//|

command AddUnitToPlatoon(punit uUnit)
{
    AddUnitToPlatoon(uUnit.GetUnitRef());
    if (state != Initialize)
    {
        OnAddNewUnitToPlatoon(uUnit);
    }
    //tutaj nie wolamy NextCommand po to zeby pluton wykonywal ta kolejke komend ktora ma nagrana
    return true;
}//|

command RemoveUnitFromPlatoon(punit uUnit)
{
    RemoveUnitFromPlatoon(uUnit.GetUnitRef());
    //tutaj nie wolamy NextCommand po to zeby pluton wykonywal ta kolejke komend ktora ma nagrana
    return true;
}//|

command DisposePlatoon()
{
    Dispose();
    return true;
}//|

command Stop()
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    PlatoonStop();
    NextCommand(1);
    state Nothing;
    return true;
}//|

command Move(int nGx, int nGy, int nLz)
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    PlatoonMove(nGx, nGy, nLz);
    SetAllMoveFlag();
    state StartMoving;
    return true;
}//|

command Attack(unit uTarget)
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    PlatoonAttack(uTarget);
    SetAllAttackFlag();
    state Attacking;
    return true;
}//|

command SetMovementMode(int nMode)
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    PlatoonSetMovementMode(nMode);
    NextCommand(1);
    return true;
}//|

//set line formation
command UserNoParam0()
{
    int nIndex, nCnt;

    nCnt = GetUnitsCount();
    if (nCnt == 0)
    {
        NextCommand(0);
        return true;
    }
    for(nIndex = 1; nIndex < nCnt; ++nIndex)
    {
        if(nIndex%2)
        {
            SetPosition(nIndex, nIndex/2+1, 0);
        }
        else
        {
            SetPosition(nIndex, -nIndex/2, 0);
        }
    }
    PlatoonMoveToFormation();
    SetAllMoveFlag();
    state StartMoving;
    return true;
}//|

//set square formation
command UserNoParam1()
{
    int nIndex, nCnt;
    int nSize;

    nCnt = GetUnitsCount();
    if (nCnt == 0)
    {
        NextCommand(0);
        return true;
    }
    nSize = sqrt(nCnt);
    if (nSize*nSize < nCnt)
    {
        ++nSize;
    }
    for(nIndex = 1; nIndex < nCnt; ++nIndex)
    {
        SetPosition(nIndex, nIndex%nSize, nIndex/nSize);
    }
    PlatoonMoveToFormation();
    SetAllMoveFlag();
    state StartMoving;
    return true;
}//|

//turn right
command UserNoParam2()
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    TurnFormation(64);
    PlatoonMoveToFormation();
    SetAllMoveFlag();
    state StartMoving;
    return true;
}//|

//turn left
command UserNoParam3()
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    TurnFormation(-64);
    PlatoonMoveToFormation();
    SetAllMoveFlag();
    state StartMoving;
    return true;
}//|

//wait for end units attacking
command UserNoParam5()
{
    if (GetUnitsCount() == 0)
    {
        NextCommand(0);
        return true;
    }
    SetAllAttackFlag();
    state StartWaitForEndAttacking;
    return true;
}//|

//set move mode
command UserOneParam0(int nMoveMode)
{
    //nie ma sprawdzania UnitsCount
    m_nMoveMode = nMoveMode;
    NextCommand(1);
    return true;
}//|

//set defend units in platoon mode
command UserOneParam1(int nMode)
{
    //nie ma sprawdzania UnitsCount
    m_bDefendUnitsInPlatoon = nMode;
    NextCommand(1);
    return true;
}//|
}

