#define BUILDER_EC

#include "Translates.ech"
#include "Items.ech"

builder TRL_SCRIPT_BUILDER
{

////    Declarations    ////

state Nothing;
state BuildBuilding;
state MovingToBuildBuilding;
state BuildElement;
state MovingToBuildElement;
state BuildElement;

#define STOPCURRENTACTION
function int StopCurrentAction();

#define RESUMEPREVIOUSSTATE
function void ResumePreviousState();

#include "Common.ech"
#include "Trace.ech"
#include "Lights.ech"
#include "Freeze.ech"
#include "Siege.ech"
#include "Move.ech"
#include "Crew.ech"
#include "Recycle.ech"
#include "Retreat.ech"

////    Functions    ////

function int StopCurrentAction()
{
    RemoveAllElements();
    if (IsBuildWorking())
    {
        CallStopMoving();//dziala jak funkcja CallStopBuild
    }
    return true;
}//|

function void ResumePreviousState()
{
    RESUME_START()
    RESUME_STATES_SIEGE()
    RESUME_STATES_MOVE()
    RESUME_STATES_CREW()
    RESUME_STATES_RETREAT()
    RESUME_STATE2(BuildBuilding, MovingToBuildBuilding)
    RESUME_STATE2(BuildElement, MovingToBuildElement)
    RESUME_END()
}//|

////    States    ////

state Nothing
{
    return Nothing;
}//|
    
state BuildBuilding
{
    if (IsBuildWorking())
    {
        return BuildBuilding;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

state MovingToBuildBuilding
{
    if (IsMoving())
    {
        TRACE1("BuildRobot->IsMoving");
        return MovingToBuildBuilding;
    }
    else
    {
        if (IsInGoodPointForCurrentBuild())
        {
            TRACE1("BuildRobot->CallBuildBuilding");
            CallBuildBuilding();
            return BuildBuilding;
        }
        else
        {
            //!!jakis licznik?
            m_nMoveToGx = GetCurrentBuildLocationX();
            m_nMoveToGy = GetCurrentBuildLocationY();
            m_nMoveToLz = GetCurrentBuildLocationZ();
            CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
            return MovingToBuildBuilding;
        }
    }
}//|

state MovingToBuildElement
{
    if (IsMoving())
    {
        return MovingToBuildElement;
    }
    else
    {
        if (IsInGoodPointForCurrentBuild())
        {
            CallBuildCurrentElement();
            return BuildElement;
        }
        else
        {
            //!!jakis licznik?
            m_nMoveToGx = GetCurrentBuildLocationX();
            m_nMoveToGy = GetCurrentBuildLocationY();
            m_nMoveToLz = GetCurrentBuildLocationZ();
            CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
            return MovingToBuildElement;
        }
    }
}//|

state BuildElement
{
    if (IsBuildWorking())
    {
        return BuildElement;
    }
    else
    {
        if (NextElementPoint())
        {
            m_nMoveToGx = GetCurrentBuildLocationX();
            m_nMoveToGy = GetCurrentBuildLocationY();
            m_nMoveToLz = GetCurrentBuildLocationZ();
            CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
            return MovingToBuildElement;
        }
        else
        {
            NextCommand(1);
            return Nothing;
        }
    }
}//|

////    Events    ////

event OnHit()
{
	if ((state == Nothing) && (GetHP() < GetMaxHP()/2))
	{
        StartRetreat();
	}
    return true;
}//|

event OnTransported(int bToNewWorld)
{
    return true;
}//|

event OnConvertedToNewPlayer()
{
    return true;
}//|

////    Commands    ////

command Initialize()
{
    SetCannonsAutoFire(true);
    return true;
}//|

command Uninitialize()
{
    ResetMoveOnHumanTarget();
    return true;
}//|

//bez nazwy - wywolywane po wybraniu miejsca
command BuildBuilding(int nX, int nY, int nZ, int nAlpha, int nID) hidden button TRL_BUILD_BUILDING
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildBuilding(nX, nY, nZ, nAlpha, nID);
    m_nMoveToGx = GetCurrentBuildLocationX();
    m_nMoveToGy = GetCurrentBuildLocationY();
    m_nMoveToLz = GetCurrentBuildLocationZ();
    CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
    state MovingToBuildBuilding;
    return true;
}//|

command BuildTrench(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) hidden button TRL_BUILD_TRENCH hotkey
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildElements(buildTrench);
    if (AddElementsLine(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMoveToGx = GetCurrentBuildLocationX();
        m_nMoveToGy = GetCurrentBuildLocationY();
        m_nMoveToLz = GetCurrentBuildLocationZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToBuildElement;
    }
    return true;
}//|

command BuildFlatTerrain(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) hidden button TRL_BUILD_FLAT hotkey
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildElements(buildFlatTerrain);
    if (AddElementsLine(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMoveToGx = GetCurrentBuildLocationX();
        m_nMoveToGy = GetCurrentBuildLocationY();
        m_nMoveToLz = GetCurrentBuildLocationZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToBuildElement;
    }
    return true;
}//|

command BuildWall(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) hidden button TRL_BUILD_WALL hotkey
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildElements(buildWall);
    if (AddElementsLine(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMoveToGx = GetCurrentBuildLocationX();
        m_nMoveToGy = GetCurrentBuildLocationY();
        m_nMoveToLz = GetCurrentBuildLocationZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToBuildElement;
    }
    return true;
}//|

command BuildWideBridge(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) hidden button TRL_BUILD_WIDEBRIDGE hotkey
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildElements(buildWideBridge);
    if (AddElementsLine(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMoveToGx = GetCurrentBuildLocationX();
        m_nMoveToGy = GetCurrentBuildLocationY();
        m_nMoveToLz = GetCurrentBuildLocationZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToBuildElement;
    }
    return true;
}//|

command BuildNarrowBridge(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) hidden button TRL_BUILD_NARROWBRIDGE hotkey
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    SetBuildTypeBuildElements(buildNarrowBridge);
    if (AddElementsLine(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMoveToGx = GetCurrentBuildLocationX();
        m_nMoveToGy = GetCurrentBuildLocationY();
        m_nMoveToLz = GetCurrentBuildLocationZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToBuildElement;
    }
    return true;
}//|
}
