#define REPAIRER_EC

#include "Translates.ech"
#include "Items.ech"

repairer TRL_SCRIPT_REPAIRER
{

////    Declarations    ////

state Nothing;
state MovingToTarget;
state Repairing;
state AfterWorking;
state MovingToStay;

unit m_uCurrTarget;
int  m_bSelfTarget;
int  m_bTargetRepairingThis;
int  m_nStayGx;
int  m_nStayGy;
int  m_nStayLz;

enum repairMode
{
    TRL_REPAIRMODE_0 item ITEM_REPAIRMODE_0 description TRL_REPAIRMODE_0_DESCR,
    TRL_REPAIRMODE_1 item ITEM_REPAIRMODE_1 description TRL_REPAIRMODE_1_DESCR,
multi:
    TRL_REPAIRMODE_X item ITEM_REPAIRMODE_X description TRL_REPAIRMODE_X_DESCR
}//|

#define STOPCURRENTACTION
function int StopCurrentAction();

#define RESUMEPREVIOUSSTATE
function void ResumePreviousState();

#include "Common.ech"
#include "Trace.ech"
#include "Lights.ech"
#include "Freeze.ech"
#include "Siege.ech"
#include "Move.ech"
#include "Crew.ech"
#include "Recycle.ech"
#include "Retreat.ech"
#include "Escort.ech"

////    Functions    ////

function void SetCurrentTarget(unit uTarget, int bSelfTarget)
{
    m_uCurrTarget = uTarget;
    m_bSelfTarget = bSelfTarget;
    if (uTarget != null)
    {
        if (m_uCurrTarget.IsRepairer() && (m_uCurrTarget.SetEventOnGetTarget() == GetUnitRef()))
        {
            m_bTargetRepairingThis = true;
        }
        else
        {
            m_bTargetRepairingThis = false;
        }
    }
    else
    {
        m_bTargetRepairingThis = false;
    }
    SetTargetObject(m_uCurrTarget);
}//|

function void ResetCurrentTarget()
{
    m_uCurrTarget = null;
    SetTargetObject(null);
}//|

function int StopCurrentAction()
{
    if (IsRepairing())
    {
        CallStopMoving();//dziala jak funkcja CallStopRepairing
    }
    ResetCurrentTarget();
    ResetEscortUnit();
    return true;
}//|

function void StopWorking()
{
    ResetCurrentTarget();
    CallStopMoving();
    NextCommand(1);
}//|

function void MoveToCalculatedTarget()
{
    if (m_bSelfTarget && m_bTargetRepairingThis)
    {
        //nie jedziemy do niego - on przyjedzie do nas
        if (m_uCurrTarget.IsRepairer() && (m_uCurrTarget.SetEventOnGetTarget() == GetUnitRef()))
        {
            m_bTargetRepairingThis = true;
        }
        else
        {
            m_bTargetRepairingThis = false;
        }
        return;
    }
    CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
}//|

function int MoveToTarget()
{
    m_nMoveToGx = GetOperateOnTargetLocationX(m_uCurrTarget);
    m_nMoveToGy = GetOperateOnTargetLocationY(m_uCurrTarget);
    m_nMoveToLz = GetOperateOnTargetLocationZ(m_uCurrTarget);
    if (IsGoodPointForOperateOnTarget(m_uCurrTarget, m_nMoveToGx, m_nMoveToGy, m_nMoveToLz))
    {
        MoveToCalculatedTarget();
        return true;
    }
    return false;
}//|

function int FindTargetToRepair()
{
    int nIndex, nTargetsCount;
    unit uNewTarget;
    
    BuildTargetsArray(findTargetWater | findTargetGround | findTargetBuilding | findTargetTank, findOurUnit | findAllyUnit, findDestinationAnyUnit, 0);
    SortFoundTargetsArray();
    nTargetsCount = GetTargetsCount();
    if (nTargetsCount > 0)
    {
        StartEnumTargetsArray();
        for (nIndex = 0; nIndex < nTargetsCount; ++nIndex)
        {
            TRACE(".");
            uNewTarget = GetNextTarget();
            if (!uNewTarget.IsFroozen() && CanBeRepaired(uNewTarget) && 
                (DistanceTo(uNewTarget.GetLocationX(), uNewTarget.GetLocationY()) < 6))
            {
                TRACE(":");
                m_nMoveToGx = GetOperateOnTargetLocationX(uNewTarget);
                m_nMoveToGy = GetOperateOnTargetLocationY(uNewTarget);
                m_nMoveToLz = GetOperateOnTargetLocationZ(uNewTarget);
                if (IsGoodPointForOperateOnTarget(uNewTarget, m_nMoveToGx, m_nMoveToGy, m_nMoveToLz))
                {
                    TRACE1("!");
                    EndEnumTargetsArray();
                    SetCurrentTarget(uNewTarget, true);
                    return true;
                }
            }
        }
        EndEnumTargetsArray();
    }
    return false;
}//|

function void ResumePreviousState()
{
    RESUME_START()
    RESUME_STATES_SIEGE()
    RESUME_STATES_MOVE()
    RESUME_STATES_CREW()
    RESUME_STATES_RETREAT()
    RESUME_STATES_ESCORT()
    RESUME_STATE(MovingToTarget)
    RESUME_STATE(Repairing)
    RESUME_STATE(AfterWorking)
    ELSE_RESUME_STATE(MovingToStay)
    {
        CallMoveToPoint(m_nStayGx, m_nStayGy, m_nStayLz);
        SET_RESUME_STATE(MovingToStay);
    }
    RESUME_END()
}//|

////    States    ////
    
state Nothing
{
    if ((repairMode == 0) && FindTargetToRepair())
    {
        m_nStayGx = GetLocationX();
        m_nStayGy = GetLocationY();
        m_nStayLz = GetLocationZ();
        MoveToCalculatedTarget();
        TRACE1("Nothing->MovingToTarget");
        return MovingToTarget;
    }
    return Nothing;
}//|

state MovingToTarget
{
    TRACE1("MovingToTarget");
    if (IsMoving())
    {
        //sprawdzanie czy cel jeszcze mozna naprawic i czy sie ruszyl (wtedy trzeba zmienic kierunek jazdy)   
        if (CanBeRepaired(m_uCurrTarget))
        {
            if (!IsGoodPointForOperateOnTarget(m_uCurrTarget, m_nMoveToGx, m_nMoveToGy, m_nMoveToLz) ||
                !IsFreePoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz))
            {
                //target has moved
                if (MoveToTarget())
                {
                    return MovingToTarget;
                }
                else
                {
                    //unable to find operation point for target
                    TRACE1("IsMoving - unable to find operation point for target");
                    StopWorking();
                    return AfterWorking, 1;
                }
            }
        }
        else
        {
            TRACE1("XXX 1");
            StopWorking();
            return AfterWorking, 1;
        }
        return MovingToTarget;
    }
    else
    {
        //sprawdzic czy w punkcie w ktorym jestesmy mozemy zaczac naprawe
        if (IsInGoodPointForOperateOnTarget(m_uCurrTarget))
        {
            if (CanBeRepaired(m_uCurrTarget))
            {
                TRACE1("->Repairing");
                CallRepair(m_uCurrTarget);
                return Repairing;
            }
            else
            {
                TRACE1("XXX 2");
                StopWorking();
                return AfterWorking, 1;
            }
        }
        else
        {
            if (MoveToTarget())
            {
                return MovingToTarget;
            }
            else
            {//unable to find operation point for target
                TRACE1("!IsMoving - unable to find operation point for target");
                StopWorking();
                return AfterWorking, 1;
            }
        }
    }
}//|

state Repairing
{
    if (IsRepairing())
    {
        return Repairing, 5;
    }
    else
    {
        if (CanBeRepaired(m_uCurrTarget) && MoveToTarget())
        {
            //z jakiegos powodu jeszcze go nie naprawilismy - odjechal ?
            return MovingToTarget;
        }
        else
        {
            StopWorking();
            return AfterWorking, 1;
        }
    }
}//|

state AfterWorking
{
    if (!m_bSelfTarget)
    {
        return Nothing, 5;
    }
    if ((repairMode == 0) && FindTargetToRepair())
    {
        MoveToCalculatedTarget();
        return MovingToTarget;
    }
    else
    {
        CallMoveToPoint(m_nStayGx, m_nStayGy, GetLocationZ());
        return MovingToStay;
    }
}//|

state MovingToStay
{
    if (IsMoving())
    {
        if ((repairMode == 0) && FindTargetToRepair())
        {
            MoveToCalculatedTarget();
            return MovingToTarget;
        }
        return MovingToStay;
    }
    else
    {
        m_bSelfTarget = false;
        return Nothing, 5;
    }
}//|

////    Events    ////

event OnCannonFoundTarget(unit uTarget)
{
    return false;
}//|

event OnHit()
{
	if ((state == Nothing) && (GetHP() < GetMaxHP()/2))
	{
        StartRetreat();
	}
    return true;
}//|

event OnTransported(int bToNewWorld)
{
    ResetCurrentTarget();
    ResetEscortUnit();
    return true;
}//|

event OnConvertedToNewPlayer()
{
    ResetCurrentTarget();
    ResetEscortUnit();
    return true;
}//|

event OnGetTarget()
{
    return m_uCurrTarget;
}//|

////    Commands    ////

command Initialize()
{
    SetCannonsAutoFire(true);
    return true;
}//|

command Uninitialize()
{
    //wykasowac referencje
    ResetCurrentTarget();
    ResetEscortUnit();
    ResetMoveOnHumanTarget();
    return true;
}//|

command Repair(unit uTarget) hidden button TRL_REPAIRMODE
{
    EXIT_COMMAND_IN_SIEGE_MODE();
    if (CanBeRepaired(uTarget))
    {
        SetCurrentTarget(uTarget, false);
        if (MoveToTarget())
        {
            state MovingToTarget;
        }
        else
        {
            ResetCurrentTarget();
            NextCommand(0);
        }
    }
    else
    {
        NextCommand(0);
    }
    return true;
}//|

command SetRepairMode(int nMode) button repairMode priority PRIOR_REPAIR advanced
{
    if (nMode == -1)
    {
        repairMode = (repairMode + 1) % 2;
    }
    else
    {
        repairMode = nMode;
    }
    NextCommand(1);
    return true;
}//|
}
