#define SAPPER_EC

#include "Translates.ech"
#include "Items.ech"

sapper TRL_SCRIPT_SAPPER
{

////    Declarations    ////

state Nothing;
state MovingToMinePutPoint;
state PuttingMine;
state WaitForMines;

int m_nMinePointGx;
int m_nMinePointGy;
int m_nMinePointLz;

#define STOPCURRENTACTION
function int StopCurrentAction();

#define RESUMEPREVIOUSSTATE
function void ResumePreviousState();

#include "Common.ech"
#include "Trace.ech"
#include "Lights.ech"
#include "Freeze.ech"
#include "Siege.ech"
#include "Move.ech"
#include "Crew.ech"
#include "Recycle.ech"
#include "Retreat.ech"

////    Functions    ////

function int StopCurrentAction()
{
    ResetMinePoints();
    return true;
}//|

function void ResumePreviousState()
{
    RESUME_START()
    RESUME_STATES_SIEGE()
    RESUME_STATES_MOVE()
    RESUME_STATES_CREW()
    RESUME_STATES_RETREAT()
    RESUME_STATE(PuttingMine)
    RESUME_STATE(WaitForMines)
    ELSE_RESUME_STATE(MovingToMinePutPoint)
    {
        CallMoveToPoint(m_nMinePointGx, m_nMinePointGy, m_nMinePointLz);
        SET_RESUME_STATE(MovingToMinePutPoint);
    }
    RESUME_END()
}//|

////    States    ////

state Nothing
{
    return Nothing;
}//|

state MovingToMinePutPoint
{
    if (IsMoving())
    {
        TRACE1("MovingToMinePutPoint");
        return MovingToMinePutPoint;
    }
    else
    {
        if ((GetLocationX() == m_nMinePointGx) && (GetLocationY() == m_nMinePointGy) && (GetLocationZ() == m_nMinePointLz))
        {
            if (HaveMines())
            {
                TRACE1("MovingToMinePutPoint -> PuttingMine");
                CallPutMine();
                return PuttingMine;
            }
            else
            {
                TRACE1("MovingToMinePutPoint -> WaitForMines");
                return WaitForMines;
            }
        }
        else
        {
            //moze zastosowac jakis licznik (po wyzerowaniu ktorego przechodzimy do nastepnego punktu) 
            //zeby w kolko nie wywolywac ponizszego
            TRACE1("MovingToMinePutPoint");
            CallMoveToPointForce(m_nMinePointGx, m_nMinePointGy, m_nMinePointLz);
            return MovingToMinePutPoint;
        }
    }
}//|

state PuttingMine
{
    if (IsPuttingMine())
    {
        //jeszcze nie skonczyl
        TRACE1("PuttingMine");
        return PuttingMine;
    }
    else
    {
        if (NextMinePoint())
        {
            m_nMinePointGx = GetCurrMinePointX();
            m_nMinePointGy = GetCurrMinePointY();
            m_nMinePointLz = GetCurrMinePointZ();
            CallMoveToPointForce(m_nMinePointGx, m_nMinePointGy, m_nMinePointLz);
            TRACE1("PuttingMine -> MovingToMinePutPoint");
            return MovingToMinePutPoint;
        }
        else
        {
            //nie ma juz wiecej punktow do zaminowania
            TRACE1("PuttingMine -> Nothing");
            NextCommand(1);
            return Nothing;
        }
    }
}//|

state WaitForMines
{
    if (HaveMines())
    {
        TRACE1("WaitForMines -> PuttingMine");
        CallPutMine();
        return PuttingMine;
    }
    else
    {
        TRACE1("WaitForMines");
        return WaitForMines;
    }
}//|

////    Events    ////

event OnCannonFoundTarget(unit uTarget)
{
    return false;
}//|

event OnHit()
{
	if ((state == Nothing) && (GetHP() < GetMaxHP()/2))
	{
        StartRetreat();
	}
    return true;
}//|

event OnTransported(int bToNewWorld)
{
    return true;
}//|

event OnConvertedToNewPlayer()
{
    return true;
}//|

////    Commands    ////

command Initialize()
{
    SetCannonsAutoFire(true);
    return true;
}//|

command Uninitialize()
{
    ResetMoveOnHumanTarget();
    return true;
}//|

command MineTerrainClose(int nX1, int nY1, int nZ1, int nX2, int nY2, int nZ2) button TRL_MINECLOSE item ITEM_MINECLOSE description TRL_MINECLOSE_DESCR hotkey priority PRIOR_MINECLOSE
{
    ResetMinePoints();
    if (AddMineAreaClose(nX1, nY1, nZ1, nX2, nY2, nZ2))
    {
        m_nMinePointGx = GetCurrMinePointX();
        m_nMinePointGy = GetCurrMinePointY();
        m_nMinePointLz = GetCurrMinePointZ();
        CallMoveToPointForce(m_nMinePointGx, m_nMinePointGy, m_nMinePointLz);
        state MovingToMinePutPoint;
    }
    else
    {
        //nie mozna postawic miny na zadnym punkcie tego obszaru
        NextCommand(0);
    }
    return true;
}//|

command SupplyAmmo() button TRL_SUPPLY item ITEM_SUPPLY description TRL_SUPPLY_DESCR hotkey priority PRIOR_SUPPLY
{
    SendSupplyRequest();
    NextCommand(1);
    return true;
}//|
}
