#define SUPPLIER_EC

#include "Translates.ech"
#include "Items.ech"

supplier TRL_SCRIPT_SUPPLIER
{

////    Declarations    ////

state Nothing;
state MovingToStayPoint;
state MovingToSupplyCenter;
state MovingToObjectForSupply;
state LoadingAmmo;
state PuttingAmmo;

unit m_uCurrSupplyCenter;
int m_bFirstMovedToAssemblyPoint;
int m_nStayGx;
int m_nStayGy;
int m_nStayLz;
int m_nMoveToObjectCounter;

#define RESUMEPREVIOUSSTATE
function void ResumePreviousState();

#include "Common.ech"
#include "Trace.ech"
#include "Lights.ech"
#include "Freeze.ech"
#include "Siege.ech"
#include "Move.ech"
#include "Crew.ech"
#include "Recycle.ech"
#include "Retreat.ech"

////    Functions    ////

function void ResumePreviousState()
{
    RESUME_START()
    RESUME_STATES_SIEGE()
    RESUME_STATES_MOVE()
    RESUME_STATES_CREW()
    RESUME_STATES_RETREAT()
    RESUME_STATE(LoadingAmmo)
    RESUME_STATE(PuttingAmmo)
    ELSE_RESUME_STATE(MovingToStayPoint)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(MovingToStayPoint);
    }
    ELSE_RESUME_STATE(MovingToSupplyCenter)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(MovingToSupplyCenter);
    }
    ELSE_RESUME_STATE(MovingToObjectForSupply)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(MovingToObjectForSupply);
    }
    RESUME_END()
}//|

function void MoveToStayPoint()
{
    //za pierwszym razem przy nowym budynku jedziemy do supplyCenter, nastepnym razem juz tam skad nas zawolal budynek
    if (!m_bFirstMovedToAssemblyPoint && (m_uCurrSupplyCenter != null))
    {
        m_bFirstMovedToAssemblyPoint = true;
        m_nMoveToGx = GetSupplyCenterAssemblyPositionX(m_uCurrSupplyCenter);
        m_nMoveToGy = GetSupplyCenterAssemblyPositionY(m_uCurrSupplyCenter);
        m_nMoveToLz = GetSupplyCenterAssemblyPositionZ(m_uCurrSupplyCenter);
    }
    else
    {
        m_nMoveToGx = m_nStayGx;
        m_nMoveToGy = m_nStayGy;
        m_nMoveToLz = m_nStayLz;
    }
    //nie kasujemy m_uCurrSupplyCenter bo potrzebne w command MoveToSupplyCenterForLoading
    CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
}//|

function void MoveToCurrentPutSupplyPosition()
{
    m_nMoveToGx = GetCurrentPutSupplyPositionX();
    m_nMoveToGy = GetCurrentPutSupplyPositionY();
    m_nMoveToLz = GetCurrentPutSupplyPositionZ();
    CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
}//|

////    States    ////

state Nothing
{
    int bIsEnd;

    if (HaveObjectsForSupply())
    {
        //kontynujemy zaopatrzenie bo nie mozna zostawic zadnego obiektu
        bIsEnd = false;
        while (!bIsEnd && !CanCurrentObjectBeSupplied())
        {
            if (!NextObjectForSupply())
            {
                bIsEnd = true;
            }
        }
        if (!bIsEnd)
        {
            MoveToCurrentPutSupplyPosition();
            m_nMoveToObjectCounter = 0;
            return MovingToObjectForSupply;
        }
        else
        {
            TRACE1("HaveObjectsForSupply->Nothing");
            return Nothing;
        }
    }
    else
    {
        return Nothing;
    }
}//|

state MovingToStayPoint
{
    TRACE1("MovingToStayPoint");
    if (IsMoving())
    {
        return MovingToStayPoint;
    }
    else
    {
        return Nothing;
    }
}//|

state MovingToSupplyCenter
{
    int nPosX, nPosY, nPosZ;

    nPosX = GetLocationX();
    nPosY = GetLocationY();
    nPosZ = GetLocationZ();
    if (IsMoving())
    {
        TRACE1("M -> SC");
        if ((nPosX == m_nMoveToGx) && (nPosY == m_nMoveToGy) && (nPosZ == m_nMoveToLz))
        {
            CallStopMoving();
        }
        return MovingToSupplyCenter;
    }
    else
    {
        TRACE1("M -> SC");
        if ((nPosX == m_nMoveToGx) && (nPosY == m_nMoveToGy) && (nPosZ == m_nMoveToLz))
        {
            TRACE1("M -> SC OK");
            CallLoadAmmo();
            return LoadingAmmo;
        }
        else
        {
            TRACE1("M -> SC Again");
            CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
            return MovingToSupplyCenter;
        }
    }
}//|
    
state MovingToObjectForSupply
{
    int nPosX, nPosY, nPosZ;
    int bIsEnd;

    if (IsMoving())
    {
        if (IsLandUnit() && IsInGoodCurrentPutSupplyLocation())
        {
            CallStopMoving();
            return MovingToObjectForSupply;
        }
        if (!CanCurrentObjectBeSupplied())
        {
            CallStopMoving();
            return MovingToObjectForSupply;
        }
        CheckCurrentPutSupplyLocation();
        if ((m_nMoveToGx != GetCurrentPutSupplyPositionX()) || (m_nMoveToGy != GetCurrentPutSupplyPositionY()))
        {
            MoveToCurrentPutSupplyPosition();
        }       
        TRACE1("state MovingToObjectForSupply 1");
        return MovingToObjectForSupply;
    }
    else
    {
        if (IsInGoodCurrentPutSupplyLocation())
        {
            TRACE1("state MovingToObjectForSupply 2");
            m_nMoveToObjectCounter = 0;
            CallPutAmmo();
            return PuttingAmmo;
        }
        else
        {
            TRACE1("state MovingToObjectForSupply 3");
            //nie jest w dobrym miejscu
            if (CanCurrentObjectBeSupplied() && (m_nMoveToObjectCounter < 5))
            {
                TRACE1("state MovingToObjectForSupply 4");
                MoveToCurrentPutSupplyPosition();
                ++m_nMoveToObjectCounter;
                return MovingToObjectForSupply;
            }
            else
            {
                TRACE1("state MovingToObjectForSupply - Can't be supplied");
                //nie mozna do niego dojechac (pod ziemia), lub zabity - biezemy nastepnego
                bIsEnd = false;
                do
                {
                    if (!NextObjectForSupply())
                    {
                        bIsEnd = true;
                    }
                }
                while (!bIsEnd && !CanCurrentObjectBeSupplied());
                m_nMoveToObjectCounter = 0;
                if (!bIsEnd)
                {
                    MoveToCurrentPutSupplyPosition();
                    return MovingToObjectForSupply;
                }
                else
                {
                    //nie ma juz obiektow do zaopatrzenia - wracamy do punktu zbiorki
                    MoveToStayPoint();
                    return MovingToStayPoint;
                }
            }
        }
    }
}//|
    
state LoadingAmmo
{
    int bIsEnd;

    if (IsLoadingAmmo())
    {
        return LoadingAmmo;
    }
    else
    {
        if (HaveObjectsForSupply())
        {
            bIsEnd = false;
            while (!bIsEnd && !CanCurrentObjectBeSupplied())
            {
                if (!NextObjectForSupply())
                {
                    bIsEnd = true;
                }
            }
            m_nMoveToObjectCounter = 0;
            if (!bIsEnd)
            {
                MoveToCurrentPutSupplyPosition();
                return MovingToObjectForSupply;
            }
            else
            {
                //nie ma obiektow do zaopatrzenia - wracamy do punktu zbiorki
                MoveToStayPoint();
                return MovingToStayPoint;
            }
        }
        else
        {
            //nie ma obiektow do zaopatrzenia - wracamy do punktu zbiorki
            MoveToStayPoint();
            return MovingToStayPoint;
        }
    }
}//|
    
state PuttingAmmo
{
    int bIsEnd;

    if (IsPuttingAmmo())
    {
        return PuttingAmmo;
    }
    else
    {
        //sprawdzic czy wrzucono amunicje do biezacego obiektu
        if (WasCurrentObjectSupplied())
        {
            //ok jedziemy do nastepnego
            bIsEnd = false;
            do 
            {
                if (!NextObjectForSupply())
                {
                    bIsEnd = true;
                }
            }
            while (!bIsEnd && !CanCurrentObjectBeSupplied());
            m_nMoveToObjectCounter = 0;
            if (!bIsEnd)
            {
                MoveToCurrentPutSupplyPosition();
                return MovingToObjectForSupply;
            }
            else
            {
                //nie ma juz obiektow do zaopatrzenia - wracamy do punktu zbiorki
                MoveToStayPoint();
                return MovingToStayPoint;
            }
        }
        else
        {
            //nie wrzucilismy zaopatrzenia do niego bo np. odjechal
            bIsEnd = false;
            while (!bIsEnd && !CanCurrentObjectBeSupplied())
            {
                if (!NextObjectForSupply())
                {
                    bIsEnd = true;
                }
            }
            m_nMoveToObjectCounter = 0;
            if (!bIsEnd)
            {
                MoveToCurrentPutSupplyPosition();
                return MovingToObjectForSupply;
            }
            else
            {
                //nie ma obiektow do zaopatrzenia - wracamy do punktu zbiorki
                MoveToStayPoint();
                return MovingToStayPoint;
            }
        }
    }
}//|

////    Events    ////

event OnCannonFoundTarget(unit uTarget)
{
    return false;
}//|

event OnHit()
{
	if ((state == Nothing) && (GetHP() < GetMaxHP()/2))
	{
        StartRetreat();
	}
    return true;
}//|

event CheckFreeSupplier()
{
    if (!HaveObjectsForSupply() && (state != MovingToSupplyCenter) && (state != LoadingAmmo))
    {
        return true;
    }
    return false;
}//|

event CanAddNewRequestDirectly()
{
    if (HaveObjectsForSupply() && ((state == MovingToObjectForSupply) || (state == PuttingAmmo)))
    {
        return true;
    }
    return false;
}//|

event CheckSupplingUnits()
{
    if ((state == MovingToObjectForSupply) || (state == PuttingAmmo))
    {
        return true;
    }
    return false;
}//|

////    Commands    ////

command Initialize()
{
    SetCannonsAutoFire(true);
    //na wszelki wypadek inicjalizacja
    m_nStayGx = GetLocationX();
    m_nStayGy = GetLocationY();
    m_nStayLz = GetLocationZ();
    return true;
}//|

command Uninitialize()
{
    ResetMoveOnHumanTarget();
    m_uCurrSupplyCenter = null;
    return true;
}//|

//komenda wydawana tylko przez budynek supplyCenter
command MoveToSupplyCenterForLoading(unit uSupplyCenter) hidden
{
    TRACE1("command MoveToSupplyCenterForLoading");
    if (!HaveObjectsForSupply() && (state != LoadingAmmo))
    {
        if ((uSupplyCenter == m_uCurrSupplyCenter) && (state == MovingToSupplyCenter))
        {
            //juz tam jedziemy
            return true;
        }
        if ((state == Moving) || (state == StartMoving) || (state == MovingToStayPoint))
        {
            m_nStayGx = m_nMoveToGx;
            m_nStayGy = m_nMoveToGy;
            m_nStayLz = m_nMoveToLz;
        }
        else
        {
            m_nStayGx = GetLocationX();
            m_nStayGy = GetLocationY();
            m_nStayLz = GetLocationZ();
        }
        if (m_uCurrSupplyCenter != uSupplyCenter)
        {
            m_bFirstMovedToAssemblyPoint = false;
        }
        m_uCurrSupplyCenter = uSupplyCenter;
        SetSupplyCenterBuilding(uSupplyCenter);
        m_nMoveToGx = GetSupplyCenterLoadPositionX();
        m_nMoveToGy = GetSupplyCenterLoadPositionY();
        m_nMoveToLz = GetSupplyCenterLoadPositionZ();
        CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        state MovingToSupplyCenter;
    }
    //komenda wewnetrzna z budynku wiec nie wolamy NextCommand
    return true;
}//|
}
