#define TRANSPORTER_EC

#include "Translates.ech"
#include "Items.ech"

transporter TRL_SCRIPT_TRANSPORTER
{

////    Declarations    ////

state Nothing;
state WaitingToExitPassengers;
state ExitingPassengers;
state StartTurningToAngle;
state TurningToAngle;
state StartMovingToSpecialDestination;
state MovingToSpecialDestination;
state StartMovingToPutBuilding;
state MovingToPutBuilding;
state PuttingBuilding;
state StartMovingToTakeBuilding;
state MovingToTakeBuilding;
state TakingBuilding;

int m_nBuildGx;
int m_nBuildGy;
int m_nBuildLz;
int m_nBuildAlpha;
int m_nBuildCount;
unit m_uRecycleBuilding;
int m_bExitSinglePassenger;

#define STOPCURRENTACTION
function int StopCurrentAction();

#define RESUMEPREVIOUSSTATE
function void ResumePreviousState();

#define ONMOVINGCALLBACK 
function void OnMovingCallback();

#define CHECKCANNONSMODE
function void CheckCannonsMode();

#include "Common.ech"
#include "Trace.ech"
#include "Lights.ech"
#include "Freeze.ech"
#include "Siege.ech"
#include "Move.ech"
#include "Crew.ech"
#include "Recycle.ech"
#include "Retreat.ech"
#include "Attack.ech"
#include "Escort.ech"

////    Functions    ////

function int StopCurrentAction()
{
    StopCannonFire();
    ResetAttackTarget();
    ResetEscortUnit();
    return true;
}//|

function void ResumePreviousState()
{
    RESUME_START()
    RESUME_STATES_SIEGE()
    RESUME_STATES_MOVE()
    RESUME_STATES_CREW()
    RESUME_STATES_RETREAT()
    RESUME_STATES_ESCORT()
    RESUME_STATE(WaitingToExitPassengers)
    RESUME_STATE(ExitingPassengers)
    RESUME_STATE(StartTurningToAngle)
    RESUME_STATE(TurningToAngle)
    ELSE_RESUME_STATE2(StartMovingToSpecialDestination, MovingToSpecialDestination)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(StartMovingToSpecialDestination);
    }
    ELSE_RESUME_STATE2(StartMovingToPutBuilding, MovingToPutBuilding)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(StartMovingToPutBuilding);
    }
    RESUME_STATE(PuttingBuilding)
    ELSE_RESUME_STATE2(StartMovingToTakeBuilding, MovingToTakeBuilding)
    {
        CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
        SET_RESUME_STATE(StartMovingToTakeBuilding);
    }
    RESUME_STATE(TakingBuilding)
    RESUME_STATES_ATTACK()
    RESUME_END()
}//|

////    States    ////

state Nothing
{
    return Nothing;
}//|

state WaitingToExitPassengers
{
    if (IsMoving())
    {
        return WaitingToExitPassengers;
    }
    else
    {
        CallExitPassengers(m_bExitSinglePassenger);
        return ExitingPassengers;
    }
}//|

state ExitingPassengers
{
    if (IsExitingPassengers())
    {
        return ExitingPassengers;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

state StartTurningToAngle
{
    return TurningToAngle;
}//|

state TurningToAngle
{
    if (IsMoving())
    {
        return TurningToAngle;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

state StartMovingToSpecialDestination
{
    return MovingToSpecialDestination;
}//|

state MovingToSpecialDestination
{
    if (IsMoving())
    {
        if ((Distance(GetLocationX(), GetLocationY(), m_nMoveToGx, m_nMoveToGy) <= 1) &&
            (GetLocationZ() == m_nMoveToLz))
        {
            //znikamy wczesniej aby nie bylo widac zatrzymujacego sie pojazdu
            EndMovingToSpecialDestination();
        }
        return MovingToSpecialDestination;
    }
    else
    {
        //NextCommand niepotrzebne bo znikamy
        EndMovingToSpecialDestination();
        return MovingToSpecialDestination;
    }
}//|

state StartMovingToPutBuilding
{
    return MovingToPutBuilding;
}//|

state MovingToPutBuilding
{
    if (IsMoving())
    {
        return MovingToPutBuilding;
    }
    else
    {
        if (IsInGoodPlaceToPutBuilding(m_nBuildGx, m_nBuildGy, m_nBuildLz, m_nBuildAlpha))
        {
            CallPutBuilding(m_nBuildGx, m_nBuildGy, m_nBuildLz, m_nBuildAlpha);
            return PuttingBuilding;
        }
        else
        {
            //m_nBuildCount - zabezpieczenie przed przypadkiem gdy nie mozna dojechac do punktu
            if ((m_nBuildCount < 10) && FindPlaceToPutBuilding(m_nBuildGx, m_nBuildGy, m_nBuildLz, m_nBuildAlpha))
            {
                ++m_nBuildCount;
                m_nMoveToGx = GetFoundPlaceToPutBuildingX();
                m_nMoveToGy = GetFoundPlaceToPutBuildingY();
                m_nMoveToLz = GetFoundPlaceToPutBuildingZ();
                CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
                return StartMovingToPutBuilding;
            }
            else
            {
                FailedPutBuilding(m_nBuildGx, m_nBuildGy, m_nBuildLz, m_nBuildAlpha);
                NextCommand(0);
                return Nothing;
            }
        }
    }
}//|

state PuttingBuilding
{
    if (IsPuttingBuilding())
    {
        return PuttingBuilding;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

state StartMovingToTakeBuilding
{
    return MovingToTakeBuilding;
}//|

state MovingToTakeBuilding
{
    if (IsMoving())
    {
        return MovingToTakeBuilding;
    }
    else
    {
        if (IsInGoodPlaceToTakeBuilding(m_uRecycleBuilding))
        {
            CallTakeBuilding(m_uRecycleBuilding);
            return TakingBuilding;
        }
        else
        {
            //m_nBuildCount - zabezpieczenie przed przypadkiem gdy nie mozna dojechac do punktu
            if ((m_nBuildCount < 10) && FindPlaceToTakeBuilding(m_uRecycleBuilding))
            {
                ++m_nBuildCount;
                m_nMoveToGx = GetFoundPlaceToTakeBuildingX();
                m_nMoveToGy = GetFoundPlaceToTakeBuildingY();
                m_nMoveToLz = GetFoundPlaceToTakeBuildingZ();
                CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
                return StartMovingToTakeBuilding;
            }
            else
            {
                FailedTakeBuilding(m_uRecycleBuilding);
                m_uRecycleBuilding = null;
                NextCommand(0);
                return Nothing;
            }
        }
    }
}//|

state TakingBuilding
{
    if (IsTakingBuilding())
    {
        return TakingBuilding;
    }
    else
    {
        NextCommand(1);
        return Nothing;
    }
}//|

////    Events    ////

event OnHit()
{
	if ((state == Nothing) && (GetHP() < GetMaxHP()/2))
	{
        StartRetreat();
	}
    return true;
}//|

event OnTransported(int bToNewWorld)
{
    ResetEscortUnit();
    return true;
}//|

event OnConvertedToNewPlayer()
{
    ResetEscortUnit();
    return true;
}//|

////    Commands    ////

command Initialize()
{
    SetCannonsAutoFire(true);
#ifdef AI_SCRIPT
    movementMode = 1;
#endif AI_SCRIPT
    return true;
}//|

command Uninitialize()
{
    ResetEscortUnit();
    ResetMoveOnHumanTarget();
    m_uRecycleBuilding = null;
    return true;
}//|

command ExitPassengers() button TRL_EXITPASSENGERS item ITEM_EXITPASSENGERS description TRL_EXITPASSENGERS_DESCR hotkey priority PRIOR_EXITPASSENGERS
{
    if (GetPassengersCount() > 0)
    {
        m_bExitSinglePassenger = false;
        if (IsMoving())
        {
            CallStopMoving();
            state WaitingToExitPassengers;
        }
        else
        {
            CallExitPassengers(m_bExitSinglePassenger);
            state ExitingPassengers;
        }
    }
    else
    {
        NextCommand(0);
    }
    return true;
}//|

command ExitSinglePassenger() hidden button TRL_EXITPASSENGERS hotkey 
{
    if (GetPassengersCount() > 0)
    {
        m_bExitSinglePassenger = true;
        if (IsMoving())
        {
            CallStopMoving();
            state WaitingToExitPassengers;
        }
        else
        {
            CallExitPassengers(m_bExitSinglePassenger);
            state ExitingPassengers;
        }
    }
    else
    {
        NextCommand(0);
    }
    return true;
}//|

command TurnByAngle(int nByAngle) hidden
{
    //nie ma EXIT_IN_SIEGE i CHECK_STOP bo to jest tylko dla transportera nowych unitow
    CallTurnToAngle(GetAngle() + nByAngle);
    state StartTurningToAngle;
    return true;
}//|

command MoveToSpecialDestination(int nGx, int nGy, int nLz) hidden
{
    //nie ma EXIT_IN_SIEGE i CHECK_STOP bo to jest tylko dla transportera nowych unitow
    m_nMoveToGx = nGx;
    m_nMoveToGy = nGy;
    m_nMoveToLz = nLz;
    CallMoveToPoint(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
    state StartMovingToSpecialDestination;
    return true;
}//|

command PutBuilding(int nGx, int nGy, int nLz, int nAlpha) hidden
{
    //nie ma EXIT_IN_SIEGE i CHECK_STOP bo to jest tylko dla transportera nowych unitow
    m_nBuildGx = nGx;
    m_nBuildGy = nGy;
    m_nBuildLz = nLz;
    m_nBuildAlpha = nAlpha;
    if (FindPlaceToPutBuilding(m_nBuildGx, m_nBuildGy, m_nBuildLz, m_nBuildAlpha))
    {
        m_nMoveToGx = GetFoundPlaceToPutBuildingX();
        m_nMoveToGy = GetFoundPlaceToPutBuildingY();
        m_nMoveToLz = GetFoundPlaceToPutBuildingZ();
    }
    else
    {
        //mimo wszystko tam jedziemy
        m_nMoveToGx = m_nBuildGx;
        m_nMoveToGy = m_nBuildGy;
        m_nMoveToLz = m_nBuildLz;
    }
    CallMoveToPointForce(m_nMoveToGx, m_nMoveToGy, m_nMoveToLz);
    m_nBuildCount = 0;
    state StartMovingToPutBuilding;
    return true;
}//|

command TakeBuilding(unit uRecycleBuilding)
{
    m_uRecycleBuilding = uRecycleBuilding;
    if (FindPlaceToTakeBuilding(m_uRecycleBuilding))
    {
        m_nMoveToGx = GetFoundPlaceToTakeBuildingX();
        m_nMoveToGy = GetFoundPlaceToTakeBuildingY();
        m_nMoveToLz = GetFoundPlaceToTakeBuildingZ();
    }
    else
    {
        //mimo wszystko tam jedziemy
        m_nMoveToGx = m_uRecycleBuilding.GetLocationX();
        m_nMoveToGy = m_uRecycleBuilding.GetLocationY();
        m_nMoveToLz = m_uRecycleBuilding.GetLocationZ();
    }
    m_nBuildCount = 0;
    state StartMovingToTakeBuilding;
    return true;
}//|
}
